ARRAY = 'array'
AUTHOR = 'author'
BEHAVIOR = 'behavior'
BOOLEAN = 'boolean'
CELL = 'cell'
CELL_REF = 'cellref'
CELL_TYPE = 'celltype'
COMMENT = 'comment'
CONTENTS = 'contents'
DATA_ORIGIN = 'dataorigin'
DESIGN = 'design'
DIRECTION = 'direction'
DOCUMENT = 'document'
E = 'e'
EDIF = 'edif'
EDIF_VERSION = 'edifversion'
EDIF_LEVEL = 'ediflevel'
EXTERNAL = 'external'
FALSE = 'false'
GENERIC = 'generic'
GRAPHIC = 'graphic'
INOUT = 'inout'
INPUT = 'input'
INSTANCE = 'instance'
INSTANCE_REF = 'instanceref'
INTEGER = 'integer'
INTERFACE = 'interface'
KEYWORD_MAP = 'keywordmap'
KEYWORD_LEVEL = 'keywordlevel'
LEFT_PAREN = '('
LIBRARY = 'library'
LIBRARY_REF = 'libraryref'
LOGICMODEL = 'logicmodel'
MASKLAYOUT = 'masklayout'
MEMBER = 'member'
METAX = 'metax'
NET = 'net'
NETLIST = 'netlist'
OUTPUT = 'output'
OWNER = 'owner'
PCBLAYOUT = 'netlist'
PORT = 'port'
PORT_REF = 'portref'
PROGRAM = 'program'
PROPERTY = 'property'
NUMBER = 'number'
NUMBER_DEFINITION = 'numberdefinition'
RENAME = 'rename'
RIGHT_PAREN = ')'
RIPPER = 'ripper'
SCHEMATIC = 'schematic'
STATUS = 'status'
STRANGER = 'stranger'
STRING = 'string'
SYMBOLIC = 'symbolic'
TECHNOLOGY = 'technology'
TIE = 'tie'
TIME_STAMP = 'timestamp'
TRUE = 'true'
UNIT = 'unit'
USER_DATA = 'userdata'
VERSION = 'version'
VIEW = 'view'
VIEW_MAP = 'viewmap'
VIEW_REF = 'viewref'
VIEW_TYPE = 'viewtype'
WRITTEN = 'written'

ARRAY_RELATED_INFO = 'arrayrelatedinfo'
DESIGNATOR = 'designator'
PORT_BUNDLE = 'portbundle'
PROTECTION_FRAME = 'protectionframe'
SYMBOL = 'symbol'
PARAMETER = 'parameter'
JOINED = 'joined'
MUST_JOIN = 'mustjoin'
WEAK_JOINED = 'weakjoined'
PERMUTABLE = 'permutable'
TIMING = 'timing'
SIMULATE = 'simulate'
WEAK_JOINED = 'weakjoined'

UNUSED = 'unused'
DC_FANIN_LOAD = 'dc_fanin_load'
DC_FANOUT_LOAD = 'dc_fanout_load'
DC_MAX_FANIN = 'dc_max_fanout'
DC_MAX_FANOUT = 'dc_max_fanin'
AC_LOAD = 'ac_load'
PORT_DELAY = 'port_delay'

OFF_PAGE_CONNECTOR = 'offpageconnector'
FIGURE = 'figure'
SECTION = 'section'
NET_BUNDLE = 'netbundle'
PAGE = 'page'
COMMENT_GRAPHICS = 'commentgraphics'
PORT_IMPLEMENTATION = 'portimplementation'
WHEN = 'when'
FOLLOW = 'follow'
LOGIC_PORT = 'logicport'
BOUNDING_BOX = 'boundingbox'

VIEW_LIST = 'viewlist'

PORT_LIST = 'portlist'
GLOBAL_PORT_REF = 'globalportref'

POINT = 'point'
MI_NO_MAX = 'minomax'